@@ -417,12 +417,15 @@ class phaseWithDockerSocket(phaseBase):
     if grep_proc.returncode != 0:
       raise Error(f'fail to read ssh_config {ssh_config_path} error, you may never done build-infra: {grep_proc.stderr}')
     ssh_tunnel_procs = []
+    http_proxy = context.vars.get('http_proxy')
     for host_name in map(lambda x: x.decode(encoding='utf-8').split(' ')[1], grep_proc.stdout.splitlines()):
       socket_path = f'{context.vars["temp_dir"]}/docker.sock@{host_name}'
       if pathlib.Path(socket_path).is_socket():
         raise Error(f'fail to create socket {socket_path}, another hive process may doing build-images/deploy-services' +
                     ' or the file has been left because previus hive process aborted suddenly')
       args = ['ssh', '-N', '-F', ssh_config_path, '-L', socket_path + ':/var/run/docker.sock', host_name]
+      if http_proxy is not None:
+        args += ['-R', context.vars['http_proxy_port'] + ':' + http_proxy]
       ssh_tunnel_procs.append((socket_path, subprocess.Popen(args)))
     try:
       super().do_one(context)
@@ -453,6 +456,11 @@ class buildVolumes(phaseBase):
     super().__init__('build-volumes', 'build volumes on hives')
     self.subject_name = 'volume'
 
+  def get_limit_targets(self, context):
+    if 'limit_server' in context.vars:
+      return ':' + context.vars['limit_server'] + ':&' + context.vars["stage"]
+    return context.vars["stage"]
+
 
 class buildNetworks(phaseBase):
   def __init__(self):
@@ -558,7 +566,7 @@ class installCollection(ansbileCommandBase):
     args_collection = ['ansible-galaxy', 'collection', 'install', '-r', context.vars['install_dir'] + '/requirements.yml',
                        '-p', context.vars['context_dir'] + '/collections']
     args_requirements_azure = ['pip', 'install', '-r', context.vars['context_dir'] +
-                               '/collections/ansible_collections/azure/azcollection/requirements-azure.txt']
+                               '/collections/ansible_collections/azure/azcollection/requirements.txt']
     # subprocess.run(args_role)
     subprocess.run(args_collection)
     subprocess.run(args_requirements_azure)
@@ -579,11 +587,15 @@ class listHosts(ansbileCommandBase):
     stage = hive_yml['stages'][stage_name]
     separate_repository = stage.get('separate_repository', True)
     number_of_hosts = stage.get('number_of_hosts', 4 if separate_repository else 3)
+    custom_hostname = stage.get('custom_hostname', 'hive')
+    hostname_pattern = r'^[a-z0-9][a-z0-9-]*[a-z0-9]$'
+    if not (re.match(hostname_pattern, custom_hostname) and (len(custom_hostname) + len(name) < 57)):
+      raise Error('custom_hostname must consist of alphanumeric and hyphens, and custom_hostname + inventory_name be up to 60 in length')
     if 'ip_address_list' in stage:
       number_of_hosts = len(stage.get('ip_address_list'))
     hosts_list = []
     for idx in range(number_of_hosts):
-      hosts_list.append(f'{stage_prefix}hive{idx}.{name}')
+      hosts_list.append(f'{stage_prefix}{custom_hostname}{idx}.{name}')
     print(" ".join(hosts_list))
 
 
@@ -618,8 +630,9 @@ class listVolumes(ansbileCommandBase):
         for service in loaded_yml['services'].values():
           volumes = service.get('volumes')
           if volumes is not None:
-            volume_name = volumes[0].get('source')
-            volumes_list.append(volume_name)
+            for volume in volumes:
+              volume_name = volume.get('source')
+              volumes_list.append(volume_name)
     print(" ".join(volumes_list))
 
 
