@@ -11,6 +11,7 @@ from ansible.plugins.inventory import BaseInventoryPlugin
 from ansible.errors import AnsibleParserError
 import ipaddress
 import os
+import re
 
 DOCUMENTATION = r'''
   name: hive_inventory
@@ -86,8 +87,8 @@ DOCUMENTATION = r'''
           description: "number of cpu of hive hosts(only available when provider is 'vagrant')"
           type: int
           default: VirtualBox default
-        root_password:
-          description: "password of root user (only available when provider is 'preapred')"
+        drbd_download_url:
+          description: "download url for drbd"
         instance_type:
           description: "the instance type of hive hosts(availabe when provider is a IaaS)"
         image_name:
@@ -108,6 +109,8 @@ DOCUMENTATION = r'''
           default: False
         kms_key_id:
           description: "key id for encryption (availabe when provider is aws)"
+        kernel_version:
+          description: "kernel version to be updated at base role"
         mirrored_disk_size:
           description: disk size of hosts for drbd mirror disk, if not specified then hive does not provision mirrord disk
           type: int (megabyte)
@@ -129,6 +132,8 @@ DOCUMENTATION = r'''
           description: disk of repository server is encrypted
           type: bool
           default: False
+        root_password:
+          description: "password of root user (only available when provider is 'preapred')"
         kickstart_config:
           description: configuration for kickstart file, availble on kickstart provider
           type: dict
@@ -255,6 +260,11 @@ class Stage:
   def add_stage_group(self):
     self.inventory.add_group(self.stage_name)
     self.inventory.set_variable(self.stage_name, 'hive_provider', self.provider)
+    custom_hostname = self.stage.get('custom_hostname', 'hive')
+    hostname_pattern = r'^[a-z0-9][a-z0-9-]*[a-z0-9]$'
+    if not (re.match(hostname_pattern, custom_hostname) and (len(custom_hostname)) + len(self.name) < 55):
+      raise AnsibleParserError('custom_hostname must consist of alphanumeric and hyphens, and custom_hostname + inventory_name be up to 55 in length')
+    self.inventory.set_variable(self.stage_name, 'hive_custom_hostname', custom_hostname)
 
   def set_subnets(self):
     self.subnets = []
@@ -276,6 +286,8 @@ class Stage:
       self.inventory.set_variable(mother_name, 'hive_dev', self.stage['dev'])
     if 'kickstart_config' in self.stage:
       self.inventory.set_variable(mother_name, 'hive_kickstart_config', self.stage['kickstart_config'])
+    if 'filestore_cidr' in self.stage:
+      self.inventory.set_variable(mother_name, 'hive_filestore_cidr', self.stage['filestore_cidr'])
     if 'subnets' not in self.stage:
       try:
         net = ipaddress.ip_network(self.stage['cidr'])
@@ -323,13 +335,21 @@ class Stage:
   def add_hives(self):
     separate_repository = self.stage.get('separate_repository', True)
     number_of_hosts = self.stage.get('number_of_hosts', 4 if separate_repository else 3)
+    custom_hostname = self.stage.get('custom_hostname', 'hive')
+    hostname_pattern = r'^[a-z0-9][a-z0-9-]*[a-z0-9]$'
+    if not (re.match(hostname_pattern, custom_hostname) and (len(custom_hostname)) + len(self.name) < 55):
+      raise AnsibleParserError('custom_hostname must consist of alphanumeric and hyphens, and custom_hostname + inventory_name be up to 55 in length')
     if 'ip_address_list' in self.stage:
       number_of_hosts = len(self.stage.get('ip_address_list'))
     for idx in range(number_of_hosts):
-      host_name = f'{self.stage_prefix}hive{idx}.{self.name}'
+      host_name = f'{self.stage_prefix}{custom_hostname}{idx}.{self.name}'
       self.inventory.add_host(host_name, group=self.stage_name)
       if 'root_password' in self.stage:
         self.inventory.set_variable(host_name, 'hive_root_password', self.stage['root_password'])
+      if 'kernel_version' in self.stage:
+        self.inventory.set_variable(host_name, 'hive_kernel_version', self.stage['kernel_version'])
+      if 'drbd_download_url' in self.stage:
+        self.inventory.set_variable(host_name, 'hive_drbd_download_url', self.stage['drbd_download_url'])
       if 'internal_cidr' in self.stage:
         self.inventory.set_variable(host_name, 'hive_internal_cidr', self.stage['internal_cidr'])
       if 'internal_cidr_v6' in self.stage:
@@ -401,5 +421,5 @@ class Stage:
         self.inventory.set_variable(host_name, 'hive_available_zone', subnet.get('available_zone', az_default))
       self.inventory.set_variable(host_name, 'hive_private_ip', next(subnet['ip_list']))
       self.inventory.set_variable(host_name, 'hive_netmask', subnet['netmask'])
-    self.inventory.set_variable('hives', 'hive_swarm_master', f'{self.stage_prefix}hive{os.getenv("HIVE_FIRST_HIVE")}.{self.name}')
-    self.inventory.add_host(f'{self.stage_prefix}hive{os.getenv("HIVE_FIRST_HIVE")}.{self.name}', group='first_hive')
+    self.inventory.set_variable('hives', 'hive_swarm_master', f'{self.stage_prefix}{custom_hostname}{os.getenv("HIVE_FIRST_HIVE")}.{self.name}')
+    self.inventory.add_host(f'{self.stage_prefix}{custom_hostname}{os.getenv("HIVE_FIRST_HIVE")}.{self.name}', group='first_hive')
